@@ -1,18 +1,18 @@
 /*
- * Copyright (c) 2009-2010 Oracle and/or its affiliates.  All rights reserved. 
- * Copyright (c) 2013 Université Bordeaux 1.  All rights reserved.
+ * Copyright © 2009-2010 Oracle and/or its affiliates.  All rights reserved.
+ * Copyright © 2013 Université Bordeaux.  All rights reserved.
+ * Copyright © 2016 Inria.  All rights reserved.
  *
  * $COPYRIGHT$
- * 
+ *
  * Additional copyrights may follow
- * 
+ *
  * $HEADER$
  */
 
 #include <private/autogen/config.h>
-#include <hwloc/autogen/config.h>
-#include <private/solaris-chiptype.h>
 #include <hwloc.h>
+#include <private/solaris-chiptype.h>
 #include <stdlib.h>
 #include <strings.h>
 
@@ -53,7 +53,9 @@ static const char* sparc_modes[] = {
     "T1",
     "T2",
     "SPARC64_VII",
-    "ROCK"
+    "ROCK",
+    "T5"
+/* needs T4, T3 and T2+ ? */
 };
 
 /*****************************************************************************
@@ -145,6 +147,10 @@ static void assign_string_value(int index, char* string_val) {
                          PICL_PROPNAMELEN_MAX) == 0) {
       dss_chip_mode = 8;
     }
+    else if (strncasecmp(string_val, "SPARC-T5",
+			 PICL_PROPNAMELEN_MAX) == 0) {
+      dss_chip_mode = 9;
+    }
   } else if (index == 8) {  /* ProcessorType */
       strncpy(&dss_chip_type[0], string_val, PICL_PROPNAMELEN_MAX);
   } else if (index == 10) { /* brand-string */
@@ -262,9 +268,9 @@ static int probe_cpu(picl_nodehdl_t node_hdl, void* dummy_arg __hwloc_attribute_
 	  if (p_info.type == PICL_PTYPE_CHARSTRING) {
             val = picl_get_propval(p_hdl, &string_val, sizeof(string_val));
             if (val == PICL_SUCCESS) {
-            }	  
+            }
 	  }
-	} 
+	}
       }
     }
 
@@ -317,10 +323,10 @@ Initializes, gets the root, then walks the picl tree looking for information
 Currently, the "core" class is only needed for OPL systems
 *****************************************************************************/
 char *hwloc_solaris_get_chip_model(void) {
-    
+
     if (called_cpu_probe) {
 	if (dss_chip_mode != MODE_UNKNOWN) { /* SPARC chip */
-	    strncpy(dss_chip_model, sparc_modes[dss_chip_mode], 
+	    strncpy(dss_chip_model, sparc_modes[dss_chip_mode],
 		    PICL_PROPNAMELEN_MAX);
 	}
     } else {
